@@ -6,6 +6,8 @@ import asyncpg
 from sqlalchemy.dialects.postgresql import pypostgresql
 from sqlalchemy.engine.interfaces import Dialect
 from sqlalchemy.sql import ClauseElement
+from sqlalchemy.sql.schema import Column
+from sqlalchemy.types import TypeEngine
 
 from databases.core import DatabaseURL
 from databases.interfaces import ConnectionBackend, DatabaseBackend, TransactionBackend
@@ -17,8 +19,11 @@ _result_processors = {}  # type: dict
 
 
 class PostgresBackend(DatabaseBackend):
-    def __init__(self, database_url: typing.Union[DatabaseURL, str]) -> None:
+    def __init__(
+        self, database_url: typing.Union[DatabaseURL, str], **options: typing.Any
+    ) -> None:
         self._database_url = DatabaseURL(database_url)
+        self._options = options
         self._dialect = self._get_dialect()
         self._pool = None
 
@@ -31,16 +36,17 @@ class PostgresBackend(DatabaseBackend):
         dialect._backslash_escapes = False
         dialect.supports_sane_multi_rowcount = True  # psycopg 2.0.9+
         dialect._has_native_hstore = True
+        dialect.supports_native_decimal = True
 
         return dialect
 
     def _get_connection_kwargs(self) -> dict:
-        options = self._database_url.options
+        url_options = self._database_url.options
 
         kwargs = {}
-        min_size = options.get("min_size")
-        max_size = options.get("max_size")
-        ssl = options.get("ssl")
+        min_size = url_options.get("min_size")
+        max_size = url_options.get("max_size")
+        ssl = url_options.get("ssl")
 
         if min_size is not None:
             kwargs["min_size"] = int(min_size)
@@ -48,6 +54,9 @@ class PostgresBackend(DatabaseBackend):
             kwargs["max_size"] = int(max_size)
         if ssl is not None:
             kwargs["ssl"] = {"true": True, "false": False}[ssl.lower()]
+
+        kwargs.update(self._options)
+
         return kwargs
 
     async def connect(self) -> None:
@@ -65,17 +74,35 @@ class PostgresBackend(DatabaseBackend):
 
 
 class Record(Mapping):
-    def __init__(self, row: tuple, result_columns: tuple, dialect: Dialect) -> None:
+    def __init__(
+        self, row: asyncpg.Record, result_columns: tuple, dialect: Dialect
+    ) -> None:
         self._row = row
         self._result_columns = result_columns
         self._dialect = dialect
-        self._column_map = {
-            column_name: (idx, datatype)
-            for idx, (column_name, _, _, datatype) in enumerate(self._result_columns)
-        }
+        self._column_map = (
+            {}
+        )  # type: typing.Mapping[str, typing.Tuple[int, TypeEngine]]
+        self._column_map_int = (
+            {}
+        )  # type: typing.Mapping[int, typing.Tuple[int, TypeEngine]]
+        self._column_map_full = (
+            {}
+        )  # type: typing.Mapping[str, typing.Tuple[int, TypeEngine]]
+        for idx, (column_name, _, column, datatype) in enumerate(self._result_columns):
+            self._column_map[column_name] = (idx, datatype)
+            self._column_map_int[idx] = (idx, datatype)
+            self._column_map_full[str(column[0])] = (idx, datatype)
 
-    def __getitem__(self, key: str) -> typing.Any:
-        idx, datatype = self._column_map[key]
+    def __getitem__(self, key: typing.Any) -> typing.Any:
+        if len(self._column_map) == 0:  # raw query
+            return self._row[tuple(self._row.keys()).index(key)]
+        elif type(key) is Column:
+            idx, datatype = self._column_map_full[str(key)]
+        elif type(key) is int:
+            idx, datatype = self._column_map_int[key]
+        else:
+            idx, datatype = self._column_map[key]
         raw = self._row[idx]
         try:
             processor = _result_processors[datatype]
@@ -88,10 +115,10 @@ class Record(Mapping):
         return raw
 
     def __iter__(self) -> typing.Iterator:
-        return iter(self._column_map)
+        return iter(self._row.keys())
 
     def __len__(self) -> int:
-        return len(self._column_map)
+        return len(self._row)
 
 
 class PostgresConnection(ConnectionBackend):
@@ -125,20 +152,17 @@ class PostgresConnection(ConnectionBackend):
             return None
         return Record(row, result_columns, self._dialect)
 
-    async def execute(self, query: ClauseElement, values: dict = None) -> typing.Any:
+    async def execute(self, query: ClauseElement) -> typing.Any:
         assert self._connection is not None, "Connection is not acquired"
-        if values is not None:
-            query = query.values(values)
         query, args, result_columns = self._compile(query)
         return await self._connection.fetchval(query, *args)
 
-    async def execute_many(self, query: ClauseElement, values: list) -> None:
+    async def execute_many(self, queries: typing.List[ClauseElement]) -> None:
         assert self._connection is not None, "Connection is not acquired"
         # asyncpg uses prepared statements under the hood, so we just
         # loop through multiple executes here, which should all end up
         # using the same prepared statement.
-        for item in values:
-            single_query = query.values(item)
+        for single_query in queries:
             single_query, args, result_columns = self._compile(single_query)
             await self._connection.execute(single_query, *args)
 
